"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const architect_1 = require("@angular-devkit/architect");
const wotan_1 = require("@fimbul/wotan");
const inversify_1 = require("inversify");
const resolve = require("resolve");
const fs_1 = require("fs");
const path = require("path");
const yaml = require("js-yaml");
async function lint(rawOptions, context) {
    if (!context.target) {
        return { success: true };
    }
    const project = [];
    for (const target of ['build', 'test']) {
        try {
            project.push((await context.getTargetOptions({ project: context.target.project, target })).tsConfig);
        }
        catch { }
    }
    context.logger.info(JSON.stringify(project));
    const globalOptionsPath = path.join(context.workspaceRoot, '.fimbullinter.yaml');
    try {
        rawOptions = { ...yaml.load(await fs_1.promises.readFile(globalOptionsPath, { encoding: 'utf-8' })), project, ...rawOptions };
        context.logger.debug(`Using global options from ${globalOptionsPath}`);
    }
    catch (e) {
        context.logger.debug(`Not using global options from ${globalOptionsPath}: ${e === null || e === void 0 ? void 0 : e.message}`);
    }
    context.logger.info(JSON.stringify(rawOptions.project) || 'none');
    context.logger.info(JSON.stringify(rawOptions));
    const options = wotan_1.parseGlobalOptions(rawOptions);
    context.logger.info(JSON.stringify(options));
    const container = new inversify_1.Container();
    container.bind(LintCommandRunner).toSelf();
    let Logger = class Logger {
        constructor() {
            this.warnings = new Set();
        }
        log(message) {
            context.logger.info(message);
        }
        warn(message) {
            if (!this.warnings.has(message)) {
                this.warnings.add(message);
                context.logger.warn(message);
            }
        }
        error(e) {
            context.logger.error((e === null || e === void 0 ? void 0 : e.message) || String(e));
        }
    };
    Logger = tslib_1.__decorate([
        inversify_1.injectable()
    ], Logger);
    container.bind(wotan_1.MessageHandler).to(Logger);
    container.bind(wotan_1.DirectoryService).toConstantValue({
        getCurrentDirectory() {
            return context.workspaceRoot;
        },
    });
    try {
        if (options.modules.length)
            container.load(...await Promise.all(options.modules.map((m) => loadModule(m, context.workspaceRoot, rawOptions))));
        container.load(wotan_1.createCoreModule(rawOptions), wotan_1.createDefaultModule());
        return {
            success: container.get(LintCommandRunner).run(options),
        };
    }
    catch (e) {
        return {
            success: false,
            error: e instanceof wotan_1.ConfigurationError ? e.message : (e === null || e === void 0 ? void 0 : e.stack) || (e === null || e === void 0 ? void 0 : e.message) || e,
        };
    }
}
async function loadModule(moduleName, from, options) {
    try {
        moduleName = await new Promise((res, rej) => resolve(moduleName, { basedir: from }, (err, resolved) => err ? rej(err) : res(resolved)));
    }
    catch (e) {
        throw new wotan_1.ConfigurationError(e.message);
    }
    const m = await Promise.resolve().then(() => require(moduleName));
    if (typeof (m === null || m === void 0 ? void 0 : m.createModule) !== 'function')
        throw new wotan_1.ConfigurationError(`Module '${moduleName}' does not export a function 'createModule'.`);
    return m.createModule(options);
}
let LintCommandRunner = class LintCommandRunner {
    constructor(runner, formatterLoader, logger, fs) {
        this.runner = runner;
        this.formatterLoader = formatterLoader;
        this.logger = logger;
        this.fs = fs;
    }
    run(options) {
        var _a;
        const formatter = new (this.formatterLoader.loadFormatter((_a = options.formatter) !== null && _a !== void 0 ? _a : 'stylish'))();
        const result = this.runner.lintCollection(options);
        let success = true;
        if (formatter.prefix !== undefined)
            this.logger.log(formatter.prefix);
        for (const [file, summary] of result) {
            if (summary.findings.some((f) => f.severity === 'error'))
                success = false;
            const formatted = formatter.format(file, summary);
            if (formatted !== undefined)
                this.logger.log(formatted);
            if (options.fix && summary.fixes)
                this.fs.writeFile(file, summary.content);
        }
        if (formatter.flush !== undefined) {
            const formatted = formatter.flush();
            if (formatted !== undefined)
                this.logger.log(formatted);
        }
        return success;
    }
};
LintCommandRunner = tslib_1.__decorate([
    inversify_1.injectable(),
    tslib_1.__metadata("design:paramtypes", [wotan_1.Runner,
        wotan_1.FormatterLoader,
        wotan_1.MessageHandler,
        wotan_1.CachedFileSystem])
], LintCommandRunner);
exports.default = architect_1.createBuilder(lint);
